<?php
// $Id: crmngp_crm.inc,v 1.30 2010/01/22 22:36:08 seanr Exp $

function _debug($var, $file = 'unknown', $line='unknown') {
  //mail('api@ngpsoftware.com', '[debug] ' . $file . ' ' . $line, print_r($var, true));
}

function crmngp_crm_adjust_soap($fields) {
  
  // Adjust array structure if necessary

  if(!isset($fields[0]))
    $fields = array($fields);

  if(count($fields) == 0) 
    $fields = array();

  return $fields;
}


/**
 * Loads contact(s) based on the passed parameters. If no parameters
 * are passed, we'll return all contacts (while respecting page number
 * and total results per page).
 *
 * @param $contact
 *   An array of key values that represents the search terms. If a
 *   contact_id is passed, we'll load and return only that contact.
 * @return $matches
 *   An array of contacts per the passed search terms and count.
 */
function crmngp_crm_contact_load($contact = NULL, $page = 1, $count_per = 50) {
  $contacts = array();

  if (isset($contact['contact_id'])) {
    $xml  = '<ngp:contactGet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
    $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
    $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
    $xml .=   '<contactID>'.$contact['contact_id'].'</contactID>';
    $xml .= '</ngp:contactGet>';

    $response = _crmngp_crm_request($xml, 'contactGet'); // returns a single contact.
    $contact  = $response['processRequestWithCredsResult']['contactGetOutput']['contact'];
    $contacts[] = _crmngp_crm_contact_field_transform($contact, 'to_standard');

    $result = $contacts;
  } else {
    $xml  = '<ngp:contactSearch xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
    $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
    $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
    $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform($contact));
    $xml .=   '<pageNumber>'.$page.'</pageNumber>';
    $xml .=   '<pageSize>'.$count_per.'</pageSize>';
    $xml .= '</ngp:contactSearch>';

    $response = _crmngp_crm_request($xml, 'contactSearch'); // returns a gigantic array of $count_per contacts.
    $response_contacts = $response['processRequestWithCredsResult']['contactSearchOutput']['matchedContact'];
    $response_count = $response['processRequestWithCredsResult']['contactSearchOutput']['count'];

    if (!empty($response_contacts)) {
      // If we get an exact result death comes from _crmngp_crm_contact_field_transform.
      if ($response_contacts['contactID']) {
        $response_contacts = array($response_contacts);
      }
      foreach ($response_contacts as $contact) { // nothing to see here!
        $contacts[] = _crmngp_crm_contact_field_transform($contact, 'to_standard');
      }
    }

    $result = array(
      'contacts' => $contacts,
      'count'    => $response_count,
    );
  }

  return $result;
}

/**
 * Adds a contact to NGP through the NGPAPI. If a contact_id
 * is passed, the contact will be updated instead of added.
 *
 * @param $contact
 *   An array of key values that represents this contact.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_contact_save($contact = NULL) {
  //dsm(func_get_args());
  if (!$contact) { return NULL; }
  if ((!$contact['first_name']) && (!$contact['last_name']) && (!$contact['email'])) { return NULL; }

  if (!empty($contact['phone_home'])) {
    $contact['phone_home'] = preg_replace('/\D/','',$contact['phone_home']);
  }
  if (!empty($contact['phone_work'])) {
    $contact['phone_work'] = preg_replace('/\D/','',$contact['phone_work']);
  }

  $xml  = '<ngp:contactSetICampaigns xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform($contact));
  $xml .= '</ngp:contactSetICampaigns>';
  
  //dsm($xml);

  //mail('gengel@citidc.com', '[debug] NGP Contact Save', print_r($xml,true));

  $response = _crmngp_crm_request($xml, 'contactSetICampaigns'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['contactSetOutput']['contact']['contactID'];
}

/**
 * Adds a contact to NGP through the NGPAPI. If a contact_id
 * is passed, the contact will be updated instead of added.
 *
 * @param $contact
 *   An array of key values that represents this contact.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_contact_save_codes($contact = NULL) {
  if (!$contact) { return NULL; }
  
  $codestoadd = _crmngp_crm_codestoadd_xml($contact['codes']);

  $xml  = '<ngp:contactWithCodesSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform($contact));
	$xml .= 	$codestoadd;
  $xml .= '</ngp:contactWithCodesSet>';

  $response = _crmngp_crm_request($xml, 'contactWithCodesSet'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['contactWithCodesSetOutput']['contact']['contactID'];
}

/**
 * Adds a contact to NGP through the NGPAPI. If a contact_id
 * is passed, the contact will be updated instead of added.
 *
 * @param $contact
 *   An array of key values that represents this contact.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_contact_save_email($contact = NULL) {
  if (!$contact) { return NULL; }

  $xml  = '<ngp:contactWithEmailSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform($contact));
  $xml .=   '<mainCode>'.$contact['maincode'].'</mainCode>';
  $xml .=   '<optIn>'.$contact['optin_email'].'</optIn>';
  $xml .= '</ngp:contactWithEmailSet>';

  $response = _crmngp_crm_request($xml, 'contactWithEmailSet'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['contactWithEmailSet']['contact']['contactID'];
}

/**
 * Adds a contact to NGP through the NGPAPI. If a contact_id
 * is passed, the contact will be updated instead of added.
 *
 * @param $contact
 *   An array of key values that represents this contact.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_contact_save_va($contact = NULL) {
  if (!$contact) { return NULL; }

  $xml  = '<ngp:contactSetVA xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform($contact));
  $xml .= '</ngp:contactSetVA>';

  $response = _crmngp_crm_request($xml, 'contactSetVA'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['contactSetVAOutput']['contact']['contactID'];
}

/**
 * Adds custom field data to a contact to NGP through the NGPAPI. If a
 * contact_id is passed, the contact will be updated instead of added.
 *
 * @param $contact
 *   An array of key values that represents this contact.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_contact_save_user_fields($contact = NULL) {
  if (!$contact) { return NULL; }

  $xml  = '<ngp:contactUserFieldsSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  foreach ($contact as $element => $value) {
    $xml .= "<$element>".htmlspecialchars($value)."</$element>";
  }
  $xml .= '</ngp:contactUserFieldsSet>';

  $response = _crmngp_crm_request($xml, 'contactUserFieldsSet'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['contactUserFieldsSetOutput']['contact']['contactID'];
}

/**
 * Deletes a contact from NGP through the NGPAPI.
 *
 * @param $contact_id
 *   The contact ID to delete.
 * @return $response
 *   The full response from the SOAP server.
 */
function crmngp_crm_contact_delete($contact_id = NULL) {
  if (!$contact_id) { return NULL; }

  $xml  = '<ngp:contactDelete xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<contactID>'.$contact_id.'</contactID>';
  $xml .= '</ngp:contactDelete>';

  return _crmngp_crm_request($xml, 'contactDelete');
}

/**
 * Adds volunteer info to a contact in NGP through the NGPAPI.
 *
 * @param $contact_id
 *   A contact ID.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_volunteer_save($contact_id = NULL, $code = NULL, $notes = NULL) {
  if (!$contact_id || !$code) { return 'error'; }

  $xml  = '<ngp:volunteerSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<contactId>'.$contact_id.'</contactId>';
  $xml .=   '<type>'.$code.'</type>';
  $xml .=   '<date>'.date('Y-m-d').'</date>';
  if ($notes) {
    $xml .=   '<notes>'.$notes.'</notes>';
  }
  $xml .= '</ngp:volunteerSet>';

  //mail('gengel@citidc.com','[debug] NGP Volunteer Codes', print_r($xml,true));

  $response = _crmngp_crm_request($xml, 'volunteerSet'); // return only the contact_id.
  return $response['processRequestWithCredsResult']['volunteerSetOutput']['volId'];
}

/**
 * Retrieves field info to a contact in NGP through the NGPAPI.
 *
 * @param $contact_id
 *   A contact ID.
 * 
 */
function crmngp_crm_fields_get($contact_id = NULL ) {
  if (!$contact_id) { return 'error'; }
  
  $xml  = '<ngp:fieldGet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<contactID>'.$contact_id.'</contactID>';
  //$xml .=   '<id>0</id>';
  $xml .= '</ngp:fieldGet>';

  $response = _crmngp_crm_request($xml, 'fieldGet');

  return $response['processRequestWithCredsResult']['fieldGetOutput'];

}


/**
 * Adds field info to a contact in NGP through the NGPAPI.
 *
 * @param $contact_id
 *   A contact ID.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_field_save($contact_id=NULL, $code=NULL, $type=NULL, $level=NULL, $description=NULL, $flag=NULL) {
  _debug($code,"$contact_id Field Save Called", "blah");

  if (!$contact_id || !$code) { 
    _debug('Vals: ' . $contact_id . ' ' . $code, 'Premature Death!', __LINE__);
    return NULL; 
  }
  
  $level = ($level) ? $level : 0;
  $flag = ($flag) ? $flag : 0;

  $xml  = '<ngp:fieldSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<field xsi:type="ngp:Field">';
  $xml .=   	'<contactID>'.$contact_id.'</contactID>';
  $xml .=   	'<date>'.date('Y-m-d').'</date>';
  $xml .=   	'<code>'.$code.'</code>';
  if($type)  $xml .=   	'<type>'.$type.'</type>'; // varchar(5), OL???
  $xml .=   	'<level>'.$level.'</level>'; // int(10)
  $xml .=   	'<description>'.$description.'</description>'; // varchar(100)
  $xml .=   	'<flag>'.$flag.'</flag>';
  $xml .=   '</field>';
  $xml .= '</ngp:fieldSet>';
  $response = _crmngp_crm_request($xml, 'fieldSet');

  _debug($xml, $code . 'field_save xml', __LINE__);
  _debug($response, $code . 'field_save response', __LINE__);

  return $response['processRequestWithCredsResult']['fieldSetOutput'];


}

/**
 * Deletes field info from a contact in NGP through the NGPAPI.
 *
 * @param $contact_id
 *   A contact ID.
 * @return $contact_id
 *   The numerical ID of this contact, new or existing.
 */
function crmngp_crm_field_delete($contact_id=NULL, $id = NULL, $date=NULL, $code=NULL) {

  if (!$contact_id || (!$id && !$code)) { 
    return NULL; 
  }

  $xml  = '<ngp:fieldDelete xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  //  $xml .=   '<field xsi:type="ngp:Field">';
  //  $xml .=   	'<contactID>'.$contact_id.'</contactID>';
  $xml .=       '<fieldid>' . $id  . '</fieldid>';
  //  $xml .=   	'<date>'.$date.'</date>';
  //  $xml .=   	'<code>'.$code.'</code>';
  //  $xml .=   '</field>';
  $xml .= '</ngp:fieldDelete>';
  $response = _crmngp_crm_request($xml, 'fieldDelete');

  _debug($xml, 'del request', __LINE__);
  _debug($response, 'del response', __LINE__);

  return $response['processRequestWithCredsResult']['fieldSetOutput'];
}

/**
 * Take a list of set codes, a list of possible codes, and make sure that
 * the set codes and *only* the set codes are saved in the backend
 * 
 * Designed for Interest Codes, in particular
 *
 * @param $contact_id
 *   a contact ID
 * @param $available_codes
 *   array of potential field codes to be deleted/saved (any field codes not in this list will be ignored)
 * @param $set_codes
 *   array of field codes to be saved
 */
function crmngp_crm_fields_update($contact_id, $available_codes = array(), $set_codes = array(), $type='OL') {

  _debug($available_codes, 'available', __LINE__);
  _debug($set_codes, 'set', __LINE__);
  _debug($type, 'type', __LINE__);

  if(!is_array($available_codes) || !is_array($set_codes))
    return null;

  // Retrieve all currently set field codes for the given contact
  $fields = crmngp_crm_fields_get($contact_id);
  $fields = $fields['field'] ;

  _debug($fields, 'fields', __LINE__);

  $fields = crmngp_crm_adjust_soap($fields);

  // Iterate over previously set fields;
  foreach($fields as $field_info) {
    $code = $field_info['code'];

    // Ignore if not in available list
    if (in_array($code, $available_codes)) {

      // If not in set list, delete from API; 
      // Otherwise, remove from set list
      if  (!in_array($code, $set_codes)) {
        
        $date = $field_info['date'];
        $date = substr($date,0,strpos($date,' '));
        $date = explode('/',$date);
        $date = $date[2] . '-' . 
          str_pad($date[0], 2, '0', STR_PAD_LEFT) . '-' . 
          str_pad($date[1], 2, '0', STR_PAD_LEFT);

        $id = $field_info['id'];

        $res = crmngp_crm_field_delete($contact_id, $id, $date, $code);

      } else {
        $index = array_search($code,$set_codes);
        unset($set_codes[$index]);

      }

    }

  }

  _debug($set_codes, 'set2', __LINE__);

  // Add remaining set codes
  foreach($set_codes as $code) {
    _debug($contact_id, 'contact id', __LINE__);
    _debug($type, 'type prefix', __LINE__);
    $res = crmngp_crm_field_save($contact_id, $code, $type);
    _debug($code . $res, '$res ' . $code, __LINE__);
  }
}

function crmngp_crm_fields_get_available($type = 'F', $prefix = '') {

  /*
  _debug($type, 'Type', __LINE__);
  _debug($prefix, 'Prefix', __LINE__);
  */

  $len = strlen($prefix);
  $filter = create_function('$item','return substr($item,0,' . $len . ') == "' . $prefix . '";');
  $fields = crmngp_crm_codes_get($type, false);
  if (is_array($fields)) {
    $filtered_fields = array_flip(array_filter(array_flip($fields), $filter));
    return $filtered_fields;
  } else {
    return NULL;
  }
  
  /*
  _debug($fields, 'Received Fields', __LINE__);
  _debug($filtered_fields, 'Filtered Fields', __LINE__);
  */

}

/**
 * Loads contributions(s) based on the reference ID.
 *
 * @param $reference
 *   Contribution reference ID
 */
function crmngp_crm_contribution_get_reference($reference = NULL) {
  if (!$reference) { return NULL; }

  $xml  = '<ngp:contributionGetByReference xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  //$xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<reference>'.$reference.'</reference>';
  $xml .= '</ngp:contributionGetByReference>';

  $response = _crmngp_crm_request($xml, 'contributionGetByReference'); // return only the amount.
  return $response['processRequestWithCredsResult']['contributionGetByReferenceOutput']['contactID'];
}

/**
 * Loads contributions(s) based on the passed parameters. Currently,
 * this function is only barely supported as the API is not yet
 * robust enough for contact-like searches. As such, the IO of this
 * function may change in the future.
 *
 * @param $contribution
 *   An array of key values that represents the search terms. Allowed
 *   keys are "date_start" and "date_end" (in YYYY-MM-DD) format.
 * @return $amount
 *   The amount accumulated during the time period specified.
 */
function crmngp_crm_contribution_load($contribution = NULL) {
  if (!$contribution) { return NULL; }

  $xml  = '<ngp:contributionChartGet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<startDate>'.$contribution['date_start'].'</startDate>';
  $xml .=   '<endDate>'.$contribution['date_end'].'</endDate>';
  $xml .= '</ngp:contributionChartGet>';

  $response = _crmngp_crm_request($xml, 'contributionChartGet'); // return only the amount.
  return $response['processRequestWithCredsResult']['contributionChartGet']['amount'];
}

/**
 * Adds a contribution to an existing contact.
 *
 * @param $contact_id
 *   The contact ID to add a note to. Must exist.
 * @param $contribution
 *   An array of key values that represents this contribution.
 *   Valid keys are "id", "date" (in YYYY-MM-DD format), "amount"
 *   (in "100.00" format), "period" (see below), "cycle", "member",
 *   "source", "type", "batch" (a number), "check", "method",
 *   "attribution" and "attribution2".
 *
 *   "period" is one of "C" (convention), "G" (general), "H" (historical),
 *   "O" (other), "P" (primary), "R" (runoff) or "S" (special). We default
 *   to general if no valid value has been passed.
 * @param
 * @return $contribution_id
 *   The numerical ID of this note.
 */
function crmngp_crm_contribution_save($contact_id = NULL, $contribution = NULL, $full_response = FALSE) {
  if (!$contact_id || !$contribution) { return NULL; }

  // set some defaults for the SOAP server.
  $contribution['id']     = $contribution['id'] ? $contribution['id'] : 0;
  $contribution['date']   = $contribution['date'] ? $contribution['date'] : date('Y-m-d');
  $contribution['cycle']  = $contribution['cycle'] ? $contribution['cycle'] : date('Y');
  $contribution['batch']  = is_numeric($contribution['batch']) ? $contribution['batch'] : 1;
  $period_values = array('C', 'G', 'H', 'O', 'P', 'R', 'S'); // see function docs for explanation.
  $contribution['period'] = in_array($contribution['period'], $period_values) ? $contribution['period'] : 'G';

  $xml  = '<ngp:contributionSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform(array('contact_id' => $contact_id)));
  $xml .=   '<contribution xsi:type="ngp:Contribution">';
  $xml .=     '<id>'.$contribution['id'].'</id>';
  $xml .=     '<date>'.$contribution['date'].'</date>';
  $xml .=     '<amount>'.$contribution['amount'].'</amount>';
  $xml .=     '<period>'.$contribution['period'].'</period>';
  $xml .=     '<cycle>'.htmlspecialchars($contribution['cycle']).'</cycle>';
  $xml .=     '<member>'.htmlspecialchars($contribution['member']).'</member>';
  $xml .=     '<source>'.htmlspecialchars($contribution['source']).'</source>';
  $xml .=     '<type>'.htmlspecialchars($contribution['type']).'</type>';
  $xml .=     '<batch>'.$contribution['batch'].'</batch>';
  $xml .=     '<check>'.htmlspecialchars($contribution['check']).'</check>';
  $xml .=     '<method>'.htmlspecialchars($contribution['method']).'</method>';
  $xml .=     '<attribution>'.htmlspecialchars($contribution['attribution']).'</attribution>';
  $xml .=     '<attribution2>'.htmlspecialchars($contribution['attribution2']).'</attribution2>';
  $xml .=     '<note>'.htmlspecialchars($contribution['note']).'</note>';
  $xml .=     '<attend>'.htmlspecialchars($contribution['attend']).'</attend>';
  $xml .=     '<referenceNumber>'.htmlspecialchars($contribution['referenceNumber']).'</referenceNumber>';
  $xml .=   '</contribution>';
  $xml .= '</ngp:contributionSet>';

  $response = _crmngp_crm_request($xml, 'contributionSet'); // return only the contribution_id.
  return $full_response ? $response : $response['processRequestWithCredsResult']['contributionSetOutput']['contributionID'];
}

/**
 * Deletes a contribution from NGP through the NGPAPI.
 * @todo THIS IS A STUB FUNCTION! NOT YET SUPPORTED.
 *
 * @param $contribution_id
 *   The contribution ID to delete.
 */
function crmngp_crm_contribution_delete($contribution_id = NULL) {
  return "This is not yet available through the NGP API.";
}

/**
 * Loads the note identified by the passed ID.
 * @todo THIS IS A STUB FUNCTION! NOT YET SUPPORTED.
 *
 * @param $note_id
 *   The note ID to load.
 */
function crmngp_crm_note_load($note_id = NULL) {
  return "This is not yet available through the NGP API.";
}

/**
 * Adds a note to an existing contact.
 *
 * @param $contact_id
 *   The contact ID to add a note to. Must exist.
 * @param $note
 *   An array of key values that represents this note. Valid
 *   keys are "noteID", "date" (in YYYY-MM-DD format), "text",
 *   "type", "member", "initials", and "deleted".
 * @return $note_id
 *   The numerical ID of this note.
 */
function crmngp_crm_note_save($contact_id = NULL, $note = NULL) {
  if (!$contact_id || !$note) { return NULL; }

  // set some defaults for the SOAP server.
  $note['noteID']  = $note['noteID'] ? $note['noteID'] : 0;
  $note['date']    = $note['date'] ? $note['date'] : date('Y-m-d');
  $note['deleted'] = $note['deleted'] ? $note['deleted'] : 0;

  $xml  = '<ngp:contactWithNotesSet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   _crmngp_crm_contact_xml(_crmngp_crm_contact_field_transform(array('contact_id' => $contact_id)));
  $xml .=   '<note xsi:type="ngp:Note">';
  $xml .=     '<contactID>'.$contact_id.'</contactID>';
  $xml .=     '<noteText>'.htmlspecialchars($note['text']).'</noteText>';
  $xml .=     '<noteID>'.$note['noteID'].'</noteID>';
  $xml .=     '<date>'.$note['date'].'</date>';
  $xml .=     '<type>'.htmlspecialchars($note['type']).'</type>';
  $xml .=     '<member>'.htmlspecialchars($note['member']).'</member>';
  $xml .=     '<initials>'.htmlspecialchars($note['initials']).'</initials>';
  $xml .=     '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=     '<deleted>'.$note['deleted'].'</deleted>';
  $xml .=     '<lastUpdated>'.date('Y-m-d').'</lastUpdated>';
  $xml .=     '<lastUserID>'.variable_get('crmngp_userid', NULL).'</lastUserID>';
  $xml .=   '</note>';
  $xml .= '</ngp:contactWithNotesSet>';

  $response = _crmngp_crm_request($xml, 'contactWithNotesSet'); // return only the note_id.
  return $response['processRequestWithCredsResult']['contactWithNotesSetOutput']['noteID'];
}

/**
 * Deletes a note from NGP through the NGPAPI. @todo This function is just
 * a wrapper around crmngp_crm_note_save() since the "delete" flag is part of
 * the entire note, not a new SOAP call like crmngp_crm_contact_delete(). As
 * such, this does NOT return the $response from the SOAP server, like
 * contact deletion does. Ultimately, this is pure laziness on my part.
 * The proper solution would be to abstract the note XML, like we've done
 * in _crmngp_crm_contact_xml(), but that's a need for another day.
 *
 * @param $note_id
 *   The note ID to delete.
 */
function crmngp_crm_note_delete($note_id = NULL) {
  if (!$note_id) { return NULL; } // buggery!

  $note = array('noteID' => $note_id, 'deleted' => 1);

  return crmngp_crm_note_save($note); // laziness!
}

/**
 * Generates the XML to represent a contact within the NGPAPI.
 *
 * @param $contact
 *   An array of valid fields for the NGPAPI in the order the
 *   API expects (as returned from _crmngp_crm_contact_field_transform).
 *   If no $contact passed, we return an empty <contact> element.
 * @return $xml
 *   The XML that represents a contact in the NGPAPI.
 */
function _crmngp_crm_contact_xml($contact = NULL) {
  $xml = '<contact xsi:type="ngp:Contact">';

  if ($contact) {
    foreach ($contact as $element => $value) {
      $xml .= "<$element>".htmlspecialchars($value)."</$element>";
    }
  }
  $xml .= '</contact>';

  return $xml;
}

/**
 * Generates the XML to represent a contact within the NGPAPI.
 *
 * @param $contact
 *   An array of valid fields for the NGPAPI in the order the
 *   API expects (as returned from _crmngp_crm_contact_field_transform).
 *   If no $contact passed, we return an empty <contact> element.
 * @return $xml
 *   The XML that represents a contact in the NGPAPI.
 */
function _crmngp_crm_codestoadd_xml($codes = NULL) {
  if (is_array($codes)) {
    $codestoadd = array();
    foreach ($codes as $key => $value) {
      if ($value != '0') {
        $codestoadd[] = $key;
      }
    }
  } else if ($codes != 0) {
    $codestoadd = $codes;
  }
  
	$xml = '<codesToAdd>';
	$xml .= (is_array($codestoadd)) ? implode(',', $codestoadd) : $codestoadd;
	$xml .= '</codesToAdd>';
  return $xml;
}


/**
 * Gets a litel of codes from the database.
 *
 * @param $type
 *   The type of codes to get from the database
 * @return $codes
 *   An array of codes
 */
function crmngp_crm_codes_get($type, $public = true) {
  $public = ($public) ? 'Public' : '';
  $xml  = '<ngp:codes'.$public.'Get xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<type>'.$type.'</type>';
  $xml .= '</ngp:codes'.$public.'Get>';

  $results = _crmngp_crm_request($xml, 'codes'.$public.'Get');
  $results = (!empty($results['processRequestWithCredsResult']['codes'.$public.'GetOutput']['code'])) ? $results['processRequestWithCredsResult']['codes'.$public.'GetOutput']['code'] : false;
  
  $codes = array();
  
  if (!empty($results['name'])) {
    $codes[$results['name']] = $results['description'];
  } else if (is_array($results)) {
    foreach ($results as $result) {
      $codes[$result['name']] = $result['description'];
    }
  }
  
  return $codes;
}


/**
 * Gets a list of codes from the database.
 *
 * @param $type
 *   The type of codes to get from the database
 * @return $codes
 *   An array of codes
 */
function crmngp_crm_volunteer_get($contact_id) {
  $xml  = '<ngp:volunteerGet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>'.variable_get('crmngp_campaignid', NULL).'</campaignID>';
  $xml .=   '<userID>'.variable_get('crmngp_userid', NULL).'</userID>';
  $xml .=   '<contactId>'.$contact_id.'</contactId>';
  $xml .= '</ngp:volunteerGet>';

  $results = _crmngp_crm_request($xml, 'volunteerGet');
  $results = $results['processRequestWithCredsResult']['volunteerGetOutput']['volunteer'];
  
  if ($results['Type']) {
    $codes[$results['Type']] = $results['Type'];
  } else if (is_array($results)) {
    foreach ($results as $result) {
      $codes[$result['Type']] = $result['Type'];
    }
  }
  
  return $codes;
}


/**
 * 
 *
 * @return $preferences
 *   An array of preferences
 */
function crmngp_crm_preferences_load() {
  $campaignid = (variable_get('crmngp_alt_campaignid', NULL)) ? variable_get('crmngp_alt_campaignid', NULL) : variable_get('crmngp_campaignid', NULL);
  $userid = (variable_get('crmngp_alt_userid', NULL)) ? variable_get('crmngp_alt_userid', NULL) : variable_get('crmngp_userid', NULL);
  
  $xml  = '<ngp:cwpPreferencesGet xmlns:ngp="http://www.ngpsoftware.com/ngpapi" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">';
  $xml .=   '<campaignID>' . $campaignid . '</campaignID>';
  $xml .=   '<userID>' . $userid . '</userID>';
  $xml .= '</ngp:cwpPreferencesGet>';

  $preferences = _crmngp_crm_request($xml, 'cwpPreferencesGet');

  /*  mail("gengel@citidc.com",'[debug] $preferences ' . __FILE__ . 'L:' . __LINE__, 
      print_r($preferences,true) . "\nblah");  */

  return $preferences['processRequestWithCredsResult']['cwpPreferences'];
}

/**
 * Sends the XML to the remote SOAP server and returns the response.
 *
 * @param $xml
 *   The XML request that will be sent inside the processRequest.
 * @param $type
 *   The type of transaction that the passed $xml represents.
 * @return $response
 *   The response from the SOAP request.
 */
function _crmngp_crm_request($xml = NULL, $type = NULL) {
  if (!$xml || !$type) { return NULL; } // validation of incoming.
  if (!in_array($type, _crmngp_crm_transaction_types())) { return NULL; }
  if (!variable_get('crmngp_crm_credentials', NULL)) { return NULL; }
  if (($type == 'cwpPreferencesGet') && variable_get('crmngp_alt_crm_credentials', NULL)) {
    $credentials = variable_get('crmngp_alt_crm_credentials', NULL);
  }
  else {
    $credentials = variable_get('crmngp_crm_credentials', NULL);
  }

  $params = array(
    'credentialString' => $credentials,
    'RequestXML' => $xml,
    'transType' => $type,
  );

  $client = new nusoap_client(variable_get('crmngp_crm_wsdlurl', NULL), "wsdl");
  if (strpos($type, 'Get')) {
    $client->decode_utf8 = false;
  }
  $result = $client->call('processRequestWithCreds', $params);

/*
  mail('gengel@citidc.com','[debug] Request Payload + Result', 
       'Request: ' . print_r($xml,true) . 
       "\n Response: " . print_r($client, true));
*/

  if ((variable_get('crmngp_debug', 0)) && (user_access('administer site configuration'))) { // Display debug info to admins if option in settings is checked.
    $debug = '<h2>Request</h2><code>' . htmlspecialchars($client->request, ENT_QUOTES) . '</code>';
    $debug .= '<h2>Response</h2><code>' . htmlspecialchars($client->response, ENT_QUOTES) . '</code>';
    
    drupal_add_js('misc/drupal.js');
    drupal_add_js('misc/collapse.js');
    
    $extended_debug = '<pre>' . htmlspecialchars($client->debug_str, ENT_QUOTES) . '</pre>';
    $fieldset = array(
      '#title' => "Extended debug output",
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#value' => $extended_debug
    );
    $debug .= theme('fieldset', $fieldset);
    
    $result_array = '<pre>' . print_r($result, 1) . '</pre>';
    $fieldset = array(
      '#title' => "Result array",
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#value' => $result_array
    );
    $debug .= theme('fieldset', $fieldset);
    
    $debug .= '<hr />'; // Separate individual requests.
    
    $fieldset = array(
      '#title' => "$type debug output",
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#value' => $debug
    );
    drupal_set_message(theme('fieldset', $fieldset));
  }

  return $result;
}

/**
 * @param $required
 *   Boolean; if set, users can't choose '<none>' as a value.
 * @return $states
 *   Returns array of state abbreviations for use with the FormsAPI.
 */
function crmngp_crm_contacttypes($required = 0) {
  $contacttypes = array(
    'I' => 'Individual',
    'O' => 'Organization',
  );

  return $required ? $contacttypes
   : array('' => '<'.t('none').'>') + $contacttypes;
}

/**
 * Transform to or from standardized field names or NGP field names.
 *
 * Under normal circumstances, we'd have made sure that our form sends the
 * right keys in the first place, but we're looking towards the future for a
 * standardized API to various CRMs, and this transformation represents those
 * "standard" key names turning into "CRM-specific" key names, and vice-versa.
 * This also serves as an indication of what data this CRM can accept, as
 * submitted keys that are not part of the transformation aren't passed along.
 *
 * For NGPAPI, the ORDER of the fields in the submitted SOAP request are
 * also important and if we're transforming TO NGP field names, the newly
 * returned array from this function will represent the proper order.
 *
 * @param $submitted_contact
 *   An array of submitted fields representing a contact.
 * @param $direction
 *   One of "to_standard" or "to_crm", defaults to 'to_crm'.
 * @return $modified_contact
 *   An array of transformed fields per $direction.
 */
function _crmngp_crm_contact_field_transform($submitted_contact = NULL, $direction = 'to_crm') {
  if (!$submitted_contact) { return NULL; }
  $transforms = array(
    'contact_id'     => 'contactID',
    'first_name'     => 'firstName',
    'last_name'      => 'lastName',
    'middle_name'    => 'middleName',
    'prefix'         => 'prefix',
    'suffix'         => 'suffix',
    'mail_name'      => 'mailName',
    'address1'       => 'address1',
    'address2'       => 'address2',
    'city'           => 'city',
    'state'          => 'state',
    'zip'            => 'zip',
    'greeting'       => 'salutation',
    'email'          => 'email',
    'phone_home'     => 'homePhone',
    'phone_work'     => 'workPhone',
    'phone_work_ext' => 'workExtension',
    'phone_mobile'   => 'mobilePhone',
    'organization'   => 'organization',
    'occupation'     => 'occupation',
    'employer'       => 'employer',
    'pac_name'       => 'pacName',
    'contact_type'   => 'contactType',
    'committee_id'   => 'committeeID',
		'notes'          => 'notes',
    'workAddress1'   => 'workAddress1',
    'workCity'       => 'workCity',
    'workState'      => 'workState',
    'workZip'        => 'workZip',
		'optin_sms'      => 'smsOptIn',
    'place_of_business' => 'placeOfBusiness',
  );

  if ($direction == 'to_standard') {
    $transforms = array_flip($transforms);
  }
  $modified_contact = array();
  foreach ($transforms as $old => $new) {
    if (!empty($submitted_contact[$old]) || ($submitted_contact[$old] === 0)) {
      $modified_contact[$new] = $submitted_contact[$old];
    }
  }

  return $modified_contact;
}

/**
 * @return $fields
 *   An array of field name and human-intended labels. This list,
 *   being intended for user consumption, does not include the
 *   following NGPAPI fields: contact_id, type, committee_id.
 */
function crmngp_crm_contact_field_labels($type = 'contactadd') {
  $form_labels = array(
    'contact_type'   => 'Contact Type',
    'first_name'     => 'First Name',
    'last_name'      => 'Last Name',
    'middle_name'    => 'Middle Name',
    'prefix'         => 'Prefix (Mr., Mrs., etc.)',
    'suffix'         => 'Suffix (Jr., III, etc.)',
    'mail_name'      => 'Mail Name',
    'address1'       => 'Address 1',
    'address2'       => 'Address 2',
    'city'           => 'City',
    'state'          => 'State',
    'zip'            => 'Zip',
    'greeting'       => 'Greeting',
    'email'          => 'Email',
    'phone_home'     => 'Phone (Home)',
    'phone_work'     => 'Phone (Work)',
    'phone_work_ext' => 'Phone (Work Extension)',
    'organization'   => 'Organization',
    'occupation'     => 'Occupation',
    'employer'       => 'Employer',
    'pac_name'       => 'PAC Name',
    'notes'          => 'Comments',
  );
  if($type == 'contribute'){
  	$form_labels['amount'] = 'Amount';
  	$form_labels['credit_card_number'] = 'Credit Card Number';
  	$form_labels['exp_year'] = 'Exp Year';
  	$form_labels['exp_month'] = 'Exp Month';
  }
  if($type == 'volunteer'){
	  $form_labels['codes'] = t(variable_get('crmngp_'.$type.'_codes_label', 'Codes'));
  }
  return $form_labels;
}

/**
 * @return $types
 *   A list of valid transaction types used in the NGP CRM API.
 */
function _crmngp_crm_transaction_types() {
  return array(
    'codesGet',
    'codesPublicGet',
    'contactGet',
    'contactDelete',
    'contactSearch',
    'contactSetICampaigns',
    'contactSetAGA',
    'contactSetVA',
    'contactUserFieldsSet',
    'contactWithCodesSet',
    'contactWithEmailSet',
    'contactWithNotesSet',
    'contribution',
    'contributionChartGet',
    'contributionGetByReference',
    'contributionSet',
    'cwpPreferencesGet',
    'fieldSet',
    'fieldDelete',
    'fieldGet',
    'omitEmailSet',
    'volunteerGet',
    'volunteerSet',
  );
}